
classdef h2t<handle
    %
    %   Quickstart:
    %
    %       1) Model system using LSmodel class (help LSmodel)
    %           
    %       2) Create the H2T object passing an LSmodel object as
    %       argument.
    %           Example: H2Tobj = h2t(LSmodelObj);
    %
    %       3) Check parameters required by the toolbox of choice using the
    %       checkParameters(toolbox) method.
    %           Example: H2Tobj.checkParameters('WIDE');
    %
    %       4) Set required parameters using the setParameters(par,value)
    %       method.
    %           Example: H2Tobj.setParameters('N', 10, 'Ts', 0.1);
    %
    %       5) Build the controller associated to the toolbox of choice
    %       using the buildController(toolbox) method.
    %           Example: ctrl = H2Tobj.buildController('WIDE')
    %
    %   For a list of available methods, use
    %       methods('h2t');
    %   then access instructions with
    %       help methodName
    
    
    %%%%%%%%%%%%%%  TODO  %%%%%%%%%%%%%%%%%%%
    %
    %   opzioni aggiuntive costruttore mpt (es. varie penalty)
    %
    %%%%%%%%%%%%%%  TODO  %%%%%%%%%%%%%%%%%%%

    % Note: <NEWTBX> tag identifies parts of the code that need to be
    % edited when adding a new toolbox
    
    
    properties (SetAccess = private, GetAccess = public)
        
        weights;                % State and input weights
        nx;                     % Total number of states
        nu;                     % Total number of inputs
        ny;                     % Total number of outputs
        N;                      % Prediction horizon
        Ts;                     % Sampling time
        Net;                    % Coupling actuators/states (WIDE tbx)
        X0;                     % Uncertainty set of initial state (WIDE tbx)
        Mc;                     % Probabilities of loosing packets (WIDE tbx)
        k;                      % Algorithm parameter (PnpMPC tbx)
        m;                      % Input types (PnpMPC tbx)
        p;                      % Output types (PnpMPC tbx)
        sdpsettings;            % Settings for YALMIP (PnpMPC tbx)
        installedTBX;           % Toolboxes detected in the Matlab Path
    end
    
    properties (SetAccess = public, GetAccess = public)
        M;                      % LSmodel object
    end
    
    properties (SetAccess = private, Hidden=true)
        setInputWeightsOK;
        setStateWeightsOK;
    end
    
    methods
        
        function obj = h2t (varargin)
            %H2T H2T object constructor
            %
            
            
            
            narginchk(1,1);
            
            % check if input is either SS or LSmodel
            if isa(varargin{1},'ss')
                error('The input augument must be of "LSModel" class.');
            end
                

            obj.M = varargin{1};
            obj.Ts = ss(varargin{1}).Ts;
            
            if any(isempty({obj.M.signal.name}))
                warning('Missing signal names. Consider naming all signals when creating LSmodel object using .InputName, .OutputName, .StateName of the SS object.');
            end

            % set number of states and inputs
            [obj.nx, obj.nu] = size(ss(varargin{1}).B);
            [obj.ny ~] = size(ss(varargin{1}).C);
            
            % create default net matrix
            obj.Net = (ss(obj.M).B~=0)';
            
            % <NEWTBX>
            % look for installed toolboxes
            i = 1;
            if exist('decLMI','file')
                obj.installedTBX{i} = 'WIDE';
                i=i+1;
            end
            if exist('pnpmpc','file')
                obj.installedTBX{i} = 'PnPMPC';
                i=i+1;
            end
            if exist('MPCController','file')
                obj.installedTBX{i} = 'MPT';
                i=i+1;
            end
            
        end
        
        
      
        % <NEWTBX> Additional parameters may be required
        function setParameters (obj, varargin)
            %SETPARAMETERS Set additional controller parameters.
            %
            % Use: setParameters(Label_1, Value_1, Label_2, Value_2, ...)
            %   where Label_i =
            %       'InputWeights'          > Weights on inputs
            %       'StateWeights'          > Weights on states
            %       'PredictionHorizon','N' > Prediction Horizon
            %       'SamplingTime','Ts'     > Sampling Time
            %       'Net'                   > Coupling matrix (WIDE tbx)
            %       'X0'                    > Initial uncertainty (WIDE tbx)
            %       'Mc'                    > Packetloss probability (WIDE tbx)
            %       'k'                     > Algorithm parameter (PnpMPC tbx)
            %       'm'                     > Input types (PnpMPC tbx)
            %       'p'                     > Output types (PnpMPC tbx)
            %
            % Weights can be passed either as scalar (same weight for all
            % variables) or as square matrix.
            %
            % To get a list of required options for a particular toolbox,
            % call checkParameters(toolbox) method.
            % 
            % Example: obj.setParameters('PredictionHorizon', 10, 'SamplingTime', 0.1)
            
            
            for i=1:nargin-1
                
                % look for input weights
                if strcmpi(varargin{i},'InputWeights')
                    
                    % if weight is scalar, multiply by identity matrix
                    if isscalar(varargin{i+1})
                        obj.weights.u = varargin{i+1}*eye(obj.nu);
                        obj.setInputWeightsOK = 1;
                        fprintf('\nInput weights updated.\n');
                    
                    % otherwise check size and set property
                    else
                        [rows, cols] = size(varargin{i+1});
                        if rows ~= cols
                            error('Input weight matrix must be square.');
                        end
                        if rows ~= obj.nu
                            error('Input weight matrix dimension error.');
                        end
                        obj.weights.u = varargin{i+1};
                        obj.setInputWeightsOK = 1;
                        fprintf('\nInput weights updated.\n');
                    end
                end
                
                % look for state weights
                if strcmpi(varargin{i},'StateWeights')
                    
                    % if weight is scalar, multiply by identity matrix
                    if isscalar(varargin{i+1})
                        obj.weights.x = varargin{i+1}*eye(obj.nx);
                        obj.setStateWeightsOK = 1;
                        fprintf('\nState weights updated.\n');
                    
                    % otherwise check size and set property
                    else
                        [rows, cols] = size(varargin{i+1});
                        if rows ~= cols
                            error('State weight matrix must be square.');
                        end
                        if rows ~= obj.nx
                            error('State weight matrix dimension error.');
                        end
                        obj.weights.x = varargin{i+1};
                        obj.setStateWeightsOK = 1;
                        fprintf('\nState weights updated.\n');
                    end
                end
                
                % look for output weights
                if strcmpi(varargin{i},'OutputWeights')
                    
                    % if weight is scalar, multiply by identity matrix
                    if isscalar(varargin{i+1})
                        obj.weights.y = varargin{i+1}*eye(obj.ny);
                        fprintf('\nOutput weights updated.\n');
                    
                    % otherwise check size and set property
                    else
                        [rows, cols] = size(varargin{i+1});
                        if rows ~= cols
                            error('Output weight matrix must be square.');
                        end
                        if rows ~= obj.ny
                            error('Output weight matrix dimension error.');
                        end
                        obj.weights.y = varargin{i+1};
                        fprintf('\nOutput weights updated.\n');
                    end
                end
                
                
                % look for prediction horizon
                if strcmpi(varargin{i},'PredictionHorizon')
                    
                    % check if prediction horizon is a scalar and a natural
                    % number
                    if ~isscalar(varargin{i+1}) 
                        error('Prediction horizon must be a scalar.');
                    end
%                     if ~ismember(varargin{i+1},1:varargin{1+1})
%                         error('Prediction horizon must be a natural number');
%                     end
                    
                    obj.N = varargin{i+1};
                    fprintf('\nPrediction horizon updated with N=%i.\n',obj.N);
                end
                if strcmpi(varargin{i},'N')
                    
                    % check if prediction horizon is a scalar and a natural
                    % number
                    if ~isscalar(varargin{i+1}) 
                        error('Prediction horizon must be a scalar.');
                    end
%                     if ~ismember(varargin{i+1},1:varargin{1+1})
%                         error('Prediction horizon must be a natural number');
%                     end
                    
                    obj.N = varargin{i+1};
                    fprintf('\nPrediction horizon updated with N=%f.\n',obj.N);
                end
                
                % look for sampling time
                if strcmpi(varargin{i},'SamplingTime')
                    
                    % check if sampling time is a positive scalar
                    if ~isscalar(varargin{i+1})
                        error('Sampling time must be a scalar.');
                    end
                    if varargin{i+1} <= 0
                        error('Sampling time must be strictly positive.');
                    end
                    
                    obj.Ts = varargin{i+1};
                    fprintf('\nSampling time updated with Ts=%f.\n',obj.Ts);
                end
                if strcmpi(varargin{i},'Ts')
                    
                    % check if sampling time is a positive scalar
                    if ~isscalar(varargin{i+1})
                        error('Sampling time must be a scalar.');
                    end
                    if varargin{i+1} <= 0
                        error('Sampling time must be strictly positive.');
                    end
                    
                    obj.Ts = varargin{i+1};
                    fprintf('\nSampling time updated with Ts=%f.\n',obj.Ts);
                end              
                
 
                % look for coupling matrix (WIDE tbx)
                if strcmpi(varargin{i},'Net')
                    
                    % check if #rows=#inputs and #columns=#states
                    [rows, cols] = size(varargin{i+1});
                    if rows ~= obj.nu || cols ~= obj.nx
                        error('Coupling matrix dimensions error');
                    end
                    
                    obj.Net = varargin{i+1};
                    fprintf('\nCoupling matrix updated.\n');
                end
                
                % look for initial state uncertainty set (WIDE tbx)
                if strcmpi(varargin{i},'X0')               
                    
                    obj.X0 = varargin{i+1};
                    fprintf('\nInitial uncertainty updated.\n');
                end
                
                % look for packetloss probability markov chain (WIDE tbx)
                if strcmpi(varargin{i},'PacketLoss')               
                    
                    % check if stucture
                    if ~isstruct(varargin{i+1})
                        error('Packetloss markov chain must be a structure.');
                    end
                    
                    obj.Mc = varargin{i+1};
                    fprintf('\nPacketloss probabilities updated.\n');
                end
                
                % look for k parameter (PnPMPC tbx)
                if strcmpi(varargin{i},'k')               
                    
                    % check if stucture
                    if ~iscell(varargin{i+1})
                        error('k must be a cell array in the form k = {kmin, kmax}.');
                    end
                    
                    obj.k = varargin{i+1};
                    fprintf('\nk parameter updated.\n');
                end
                
                % look for input types (PnPMPC tbx)
                if strcmpi(varargin{i},'m')               
                    
                    % check if length matches number of inputs
                    if length(varargin{i+1}) ~= obj.nu
                        error('Input types vector length must match number of inputs.');
                    end
                    
                    obj.m = varargin{i+1};
                    fprintf('\nInput types updated.\n');
                end
                
                % look for output types (PnPMPC tbx)
                if strcmpi(varargin{i},'p')               
                    
                    % check if length matches number of inputs
                    if length(varargin{i+1}) ~= obj.ny
                        error('Output types vector length must match number of outputs.');
                    end
                    
                    obj.p = varargin{i+1};
                    fprintf('\nOutput types updated.\n');
                end
                
                % look for sdpsettings (PnPMPC tbx)
                if strcmpi(varargin{i},'sdpsettings')               
                    
                    obj.sdpsettings = varargin{i+1};
                    fprintf('\nSdpsettings updated.\n');
                end
            end
            
        end
        
        % <NEWTBX> add check for new toolbox parameters
        function checkParameters (obj, tbx)
            %CHECKPARAMETERS Check prameters required by the selected
            %toolbox
            %
            % Use: checkParameters(toolbox)
            % 
            % Example: obj.checkParameters('WIDE')
            
            if strcmpi(tbx,'WIDE')
                checkParametersWIDE(obj);
            end
            if strcmpi(tbx,'MPT')
                checkParametersMPT(obj);
            end
            if strcmpi(tbx,'PnPMPC')
                checkParametersPNPMPC(obj);
            end
        end
        
        function [lower, upper] = getLimits(obj, type)
            %GETLIMITS Get limits for selected set of variables
            %
            % Use: [lower, upper] = getLimits(type)
            %   where type = {'Input', 'State', 'Output'}
            % 
            % Example: [umin, umax] = obj.getLimits('Input')
            
            % check for input limits
            if strcmpi(type, 'Input')
                lower = zeros(obj.nu, 1);
                upper = zeros(obj.nu, 1);
                for i=1:obj.nu
                    signal_index = find(strcmp(ss(obj.M).InputName{i},{obj.M.signal.name}));
                    lower(i) = obj.M.signal(signal_index).limit.min;
                    upper(i) = obj.M.signal(signal_index).limit.max;
                end
            end
            
            % check for state limits
            if strcmpi(type, 'State')
                lower = zeros(obj.nx, 1);
                upper = zeros(obj.nx, 1);
                for i=1:obj.nx
                    signal_index = find(strcmp(ss(obj.M).StateName{i},{obj.M.signal.name}));
                    lower(i) = obj.M.signal(signal_index).limit.min;
                    upper(i) = obj.M.signal(signal_index).limit.max;
                end
            end
            
            % check for output limits
            if strcmpi(type, 'Output')
                lower = zeros(obj.ny, 1);
                upper = zeros(obj.ny, 1);
                for i=1:obj.ny
                    signal_index = find(strcmp(ss(obj.M).OutputName{i},{obj.M.signal.name}));
                    lower(i) = obj.M.signal(signal_index).limit.min;
                    upper(i) = obj.M.signal(signal_index).limit.max;
                end
            end
           
            
        end
        
        % <NEWTBX> new wrappers
        function ctrl = buildController(obj, tbx)
            %BUILDCONTROLLER Build controller with the selected toolbox
            %
            % Use: controller = buildController(toolbox)
            % 
            % Example: ctrl = obj.buildController('WIDE')
            if strcmpi(tbx,'WIDE')
                ctrl = exportToWIDE(obj);
            end
            if strcmpi(tbx,'MPT')
                ctrl = exportToMPT(obj);
            end
            if strcmpi(tbx,'PnPMPC')
                ctrl = exportToPNPMPC(obj);
            end
        end
    end
    
end

